#!/usr/bin/perl

# suprnova light 2 - create your own suprnova mirror
# Copyright (C) 2004 ntworks.net
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details:
# http://www.gnu.org/copyleft/gpl.html
#
# getmirrors.pl is part of the suprnova light 2 scripts.
# Get the latest version at http://suprnova.ntworks.net/
# Send patches and comments to snlight2@ntworks.net

use strict;
use LWP::UserAgent;
use Time::HiRes qw (usleep);

$| = 1;

my ($suprnova, $requests, $delay, $file, $proxy) = ('http://suprnova.org/', 10, 500, 'snmirrors.txt', '');

while (my $clp = shift) {
    if ($clp eq '-n') {
        $requests = shift;
        $requests = 10 if (($requests !~ /\d+/) || ($requests < 1));
    }
    elsif ($clp eq '-d') {
        $delay = shift;
        $delay = 500 if ($delay !~ /\d+/);
    }
    elsif ($clp eq '-f') { $file = shift; }
    elsif ($clp eq '-p') { $proxy = shift; }
}

my %mirrors;
my ($newMirrors, $oldMirrors, $mirrors) = (0, 0, 0);

if (-f $file) {
    open (IN, "< $file") || die "could not open $file for input\n";
    while (<IN>) {
        chomp (my $mirror = $_);
        if (/^http.*/) {
            $mirrors {$mirror} = 1;
            $oldMirrors++;
        }
    }
    close IN;
}

for (my $i = 1; $i <= $requests; $i++) {
    my $content = getPage ($suprnova);
    print ("couldn't retrieve $suprnova on request #$i\n") unless defined $content;
    if ($content =~ /<frame name='main' src='(.*)'.*>/) {
        if (! $mirrors {$1}) {
            $mirrors {$1} = 1;
            $newMirrors++;
            print "new mirror found: $1\n";
        }
        else { print "old mirror found: $1\n"; }
    }
    usleep ($delay * 1000);
}

if (keys %mirrors) {
    open (OUT, "> $file") || die "could not open $file for output\n";
    foreach (keys %mirrors) {
        $mirrors++;
        print OUT "$_\n";
    }
    close OUT;
    print "$oldMirrors old mirrors, $newMirrors new mirrors, $mirrors mirrors\n";
}
else { print "no mirrors found\n"; }

sub getPage {
    my $url = shift || return -1;
    my $ua = LWP::UserAgent -> new (
        agent      => 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.7.5) Gecko/20041109 Firefox/1.0',
        timeout    => 20,
        keep_alive => 1,
    );
    $ua -> proxy ('http', $proxy) if ($proxy ne '');
    my $response = $ua -> get ($url);
    if ($response -> is_error) { return -1; }
    else { return $response -> content; }
}
